#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <iostream>
using std::cout;
using std::endl;
using namespace Eigen;
#include <QXmlStreamWriter>



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    vectorTable = ui->tableWidget_vectors;
    vectorTable->setColumnCount(4);
    QStringList headers;
    headers += QString("c");
    headers += QString("v");
    headers += QString("rhs");
    headers += QString("x");
    vectorTable->setHorizontalHeaderLabels(headers);

    // fill the table with default values
    createDefaultEquation( ui->spinBox_size->value() );

}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::createDefaultEquation(int N )
{
    vectorTable->setRowCount(N);
    for(int i=0; i<N; i++ )
    {
        // default: unconstrained
        int constrained = 0;
        double value = 0.;

        // constraints
        if( i==0  ){
            constrained = 1;
            value=-100.0;
        }
        else if( i==N/2  ){
            constrained = 1;
            value= 100.0;
        }
        else if( i==N-1  ){
            constrained = 1;
            value= 20.0;
        }

        QTableWidgetItem *item = new QTableWidgetItem(tr("%1").arg(constrained));
        vectorTable->setItem(i,0,item);

        item = new QTableWidgetItem(tr("%1").arg(value));
        vectorTable->setItem(i,1,item);
    }

}

void MainWindow::solveEquation()
{

    int N = ui->spinBox_size->value();
    if( N!=vectorTable->rowCount() )
    {
        createDefaultEquation(N);
    }



    // Setup the constraints
    Eigen::VectorXi constrained(N);      ///< boolean vector to mark where the constraints are
    Eigen::VectorXd constraintValue(N);  ///< constraint values
    for(int i=0; i<N; i++ )
    {
        constrained(i) =      vectorTable->item(i,0)->data(Qt::DisplayRole).toInt();
        constraintValue(i) =  vectorTable->item(i,1)->data(Qt::DisplayRole).toDouble();
    }


    // Create the equation by summing two equations: a simple laplacian for smoothness, and  simple constraints.
    // The simple constraint equation is multiplied by a large number to make it prioritary

    SparseMatrix<double> A(N, N); ///< system matrix
    VectorXd rhs(N);              ///< right-hand side of the equation
    double omega = 1e8;           ///< importance of the  constraint with respect to the smoothness constraint
    for(int i=0; i<N; i++)
    {
        // Off-diagonal entries of the tridiagonal matrix
        if(i>0)    A.insert(i,i-1) = -1.0;
        if(i<N-1)  A.insert(i,i+1) = -1.0;
        // Diagonal entries
        if( i==0 || i==N )
            A.insert(i,i) = 1 + omega*constrained[i];
        else
            A.insert(i,i) = 2 + omega*constrained[i];

        // Fill the right-hand side
        rhs[i] = omega*constrained[i]*constraintValue[i];
    }


    // Solve the equation
    SimplicialLLT<SparseMatrix<double> > llt(A);  ///< Cholesky decomposition of the matrix
    VectorXd x;                                   ///< solution of the equation
    x = llt.solve(rhs);


    // update the display
    for(int i=0; i<N; i++ )
    {
        QTableWidgetItem * item = new QTableWidgetItem(tr("%1").arg(rhs(i)));
        vectorTable->setItem(i,2,item);

        item = new QTableWidgetItem(tr("%1").arg(x(i)));
        vectorTable->setItem(i,3,item);
    }

}




